/*
    Copyright (C) 2002 Terratron Technologies Inc.  All Rights Reserved.
    Author: Steven Osman
 
    This file is part of gsvnc.

    gsvnc is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    gsvnc is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with gsvnc; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
    For more information visit:
    http://window.terratron.com/~sosman/ps2linux/
    http://www.terratron.com/
*/

#ifndef __GSVNC_H
#define __GSVNC_H

#ifndef __KERNEL__
#include <stdio.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#endif

// The number of frames to skip betwen updates of the image
#define GSVNC_FRAME_SKIP_COUNT 60

// Port to listen on
#define LISTEN_PORT 5900

// How often to search for updates in microseconds.  We'll set it to
// one tenth of a second
#define POLL_TIMER 100000

#define GSVNC_SHM_KEY 0x5A11CE

#define GSVNC_MAX_WIDTH 640
#define GSVNC_MAX_HEIGHT 512

#define GSVNC_CELL_WIDTH 16
#define GSVNC_CELL_HEIGHT 16


#define GSVNC_BYTES_PER_PIXEL 3

#define GSVNC_ALIGN_POINTER(x,y) {while (((unsigned int) x) % y) x++;}

// This is the version number of the application
#define GSVNC_VERSION "gsvnc-0.1.3"

// The signature should remain constant so long as the structure of
// the shared memory area doesn't change
#define GSVNC_SIGNATURE "gsvnc-0.1.3"
#define GSVNC_SIGNATURE_SIZE 64

typedef enum {
  GSVNC_BUFFER_RENDER=0,
  GSVNC_BUFFER_DISPLAY,
  GSVNC_BUFFER_FIRST,
  GSVNC_BUFFER_SECOND,
  GSVNC_BUFFER_Z_BUFFER,
  GSVNC_BUFFER_ANY_PAGE,
} gsvncBufferTypes_t;

typedef struct {
  char acSignature[GSVNC_SIGNATURE_SIZE];

  // Actual data starts on the first 16-byte alignment of this area.
  // Take a pointer of it and use GSVNC_ALIGN_POINTER to fix it.
  unsigned char aucFrameBufferDataArea[GSVNC_MAX_WIDTH*
				      GSVNC_MAX_HEIGHT*
				      GSVNC_BYTES_PER_PIXEL+1024];
  int iWidth, iHeight;
  int iUpdated;
  int iModuleLoaded;

  // Client reference counter should be used to start/stop the screen
  // captures.  If the counter is 0, the module will suspend screen
  // captures.  If it is non-zero, it will capture.  Whenever a client
  // that desires screen captures connects, it should bump up the
  // reference counter, and decrement it when connections are no longer
  // desired.
  int iClientReferenceCounter;

  int iSystemPaused;
  int iCapturePaused;
  gsvncBufferTypes_t iRequestedBuffer;
  // If iRequestedBuffer==GSVNC_ANY_PAGE, iRequestedPage will have the first
  // page to dump.
  int iRequestedPage;

  // If set to 1, the value of the z buffer will be >>'ed iClampBits
  // and then clamped to 0xff < val < 0
  int iZBufferClamping;
  int iClampBits;

  // This is where the kernel can put the ps2_image structure
  int aiTempBuffer[32];

  // 1K of data for future use.  This will make shared memory areas allocated
  // by old versions of gsvnc work with newer versions of gsvnc.
  int aiReservedForFutureUse[256];
} frameBufferData;

#ifndef __KERNEL__
/**
 * This method gets the shared memory area to which the frame buffer should be
 * stored.  It returns null on error.
 */
static inline frameBufferData *_gsvncGetSharedFrameBuffer(void) {
  frameBufferData *pFrameBuffer;
  int iShmArea;

  iShmArea=shmget(GSVNC_SHM_KEY,sizeof(frameBufferData),IPC_CREAT | 0777);
  
  if (iShmArea<0) {
    perror("GSVNC: error creating/getting shared memory area.\n");
    return 0;
  }

  pFrameBuffer=(frameBufferData *) shmat(iShmArea,0,0);

  if (!pFrameBuffer) {
    perror("GSVNC: error attatching to shared memory area.\n");
    return 0;
  } else {
    // We have our shared memory segment.
    // Finally, we'd like to see if it has been initialized yet or not.
    // If not, we want to initialize it.
    if (strcmp(pFrameBuffer->acSignature,GSVNC_SIGNATURE)) {
      // Set signature
      memset(pFrameBuffer->acSignature,0,GSVNC_SIGNATURE_SIZE);
      strcpy(pFrameBuffer->acSignature,GSVNC_SIGNATURE);

      pFrameBuffer->iUpdated=0;
      pFrameBuffer->iSystemPaused=0;
      pFrameBuffer->iCapturePaused=0;
      pFrameBuffer->iRequestedBuffer=GSVNC_BUFFER_FIRST;
      pFrameBuffer->iModuleLoaded=1;
      pFrameBuffer->iRequestedPage=0;
      pFrameBuffer->iZBufferClamping=0;
      pFrameBuffer->iClampBits=8;

      // Set width,height=-1
      pFrameBuffer->iWidth=-1;
      pFrameBuffer->iHeight=-1;

      // Set reference Counter=0
      pFrameBuffer->iClientReferenceCounter=0;

      printf("GSVNC: Created and initialized shared memory area.\n");
    }
  }

  return pFrameBuffer;
}
#endif

#endif
