/**
 * gsvnc_sps2.h by Sauce (Steven Osman)
 *
 * This file has the implementation for the sps2 portion of the gsvnc package.
 * The functions in this file and its gsvnc dependencies (currently only
 * gsvnc.h and gsvnc_sps2.c) may be used freely so long as the sps2 license is
 * adhered to.
 *
 * Note that this code is enabled by default in applications that use 
 * libsps2util and can be added by hand to other applications.
 *
 * To use gsvnc, note the following two assumptions about the current
 * implementation:
 *  1. The Z-Buffer is 24 bit
 *  2. The other buffers (render, display) are 32 bit PSMCT32
 *
 * In your code:
 * 1. #include <gsvnc_sps2.h>
 * 2. during initialization, call gsvncSPS2Init
 * 3. Every frame, call the update function
 *    a. if you're using libsps2Util, use gsvncSPS2UtilUpdate(iUpdateRate)
 *    b. if you're using the sample framework from sps2, use
 *       gsvncSPS2SampleFrameworkUpdate
 *    c. otherwise use gsvncSPS2Update
 * Then include gsvnc_sps2.c in your build process.
 *
 *
 * Note the following keys that a user can press from the VNC viewer:
 * space to pause/resume updates
 * control+p to pause/resume your application
 * "1" to switch to the first buffer
 * "2" to switch to the second buffer
 * "r" to switch to the render buffer (typically first one or second one)
 * "d" to switch to the display buffer (typically the other of the two buffers)
 *
 * "o" to display "other" pages.
 * When displaying "other" pages, use:
 * Up/Down to go back/forward one page
 * Page Up/Page Down to go back/forward 10 pages
 * Control+Page Up/Control+Page down to go back/forward 100 pages
 *
 * "z" to switch to the z buffer
 * When viewing the Z buffer,
 * "c" to toggle 8-bit clamping of the 24 bit z buffer
 * left and right arrows to shift the 8 bit clamp range of the Z value
 */

#ifndef GSVNC_SPS2_H
#define GSVNC_SPS2_H

#ifdef __cplusplus
extern "C" {
#endif

  /**
   * Initializes gsvnc for sps2.
   *
   * iUseScratchpad is non-zero if you want gsvnc to use the scratch pad.
   *   Note that gsvnc will preseve the contents of the scratch pad so the
   *   only reason not to use it is if you believe you will be accessing the
   *   scratch pad in another thread while gsvnc is updating (extremely 
   *   unlikely)
   * Returns 0 on success
   */
  int gsvncSPS2Init(int iUseScratchPad);

  /**
   * Request an update.
   * iWidth is the screen width in pixels
   * iHeight is the scren height in pixels
   * iUpdateRate is the frequency by which frames should be grabbed.  (if 
   *    iUpdateRate = 15, only every 15th call will yield a transfer unless
   *    the user has changed parameters.
   * uiFirstBuffer, uiSecondBuffer, uiZBuffer, uiRenderBuffer and 
   * uiDisplayBuffer are all page numbers of buffers.  
   * Except for uiZBuffer which should point to a 24 bit Z buffer, all other
   * buffers should be 32 bit.  You "could" point them to some other part of
   * GS memory you want to view.
   */
  int gsvncSPS2Update(int iWidth, int iHeight, int iUpdateRate, 
		      unsigned int uiFirstBuffer, unsigned int uiSecondBuffer,
		      unsigned int uiZBuffer,
		      unsigned int uiRenderBuffer,
		      unsigned int uiDisplayBuffer);

  /**
   * If you're using libsps2util, then just call this macro and pass it an
   * update rate, it'll take care of everything for you.
   */
  #define gsvncSPS2UtilUpdate(iUpdateRate) \
     gsvncSPS2Update(sps2UScreenGetWidth(), sps2UScreenGetHeight(), \
                     iUpdateRate, \
                     sps2UScreenGetDrawBuff1(), sps2UScreenGetDrawBuff2(), \
		     sps2UScreenGetZPtr(), sps2UScreenGetDrawBuffCurrent(), \
		 sps2UScreenGetDrawBuffCurrent()==sps2UScreenGetDrawBuff1() ? \
		     sps2UScreenGetDrawBuff2() : sps2UScreenGetDrawBuff1());

  /**
   * If you're using the framework that's provided with the sps2 samples,
   * then you can simply call this macro, pass it a pointer to your Screen
   * class, and you're done.
   * Note, the weird ? statement at the bottom is to pick the display buffer,
   * which is basically "not the render buffer."
   */
  #define gsvncSPS2SampleFrameworkUpdate(pScreen, iUpdateRate) \
     gsvncSPS2Update(pScreen->GetWidth(), pScreen->GetHeight(), iUpdateRate, \
                     pScreen->GetDrawBuff1(), pScreen->GetDrawBuff2(), \
                     pScreen->GetZptr(), pScreen->GetDrawFptr(), \
		     pScreen->GetDrawFptr()==pScreen->GetDrawBuff1() ? \
		     pScreen->GetDrawBuff2() : pScreen->GetDrawBuff1())

		     

#ifdef __cplusplus
}
#endif

#endif /* GSVNC_SPS2_H */
