/**
 * SPS2 Hello World
 * Copyright (C) 2002 Terratron Technologies Inc.
 * All rights reserved.
 *
 * Author: Steven "Sauce" Osman
 *
 * This program illustrates a simple Hello World application
 * using the SPS2 library.  It performs the following:
 *
 * - Initializes the SPS2 library
 * - Maps memory through the SPS2 library
 * - Copies the text "Hello World" using a DMA transfer to the
 *   scratch pad memory
 * - Displays the text from the scratch pad memory
 * - Sets the background color to blue
 */

#include <stdio.h>
#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2regstructs.h>

// This is the string we will be outputting
#define OUTPUT_STRING "Hello SPS2 world!\n"

// This is the number of Q-Words (i.e. 16 byte increments) to copy
// We want to round up to the next Q-Word
#define OUTPUT_STRING_QWC ((strlen(OUTPUT_STRING)+15) >> 4)

int main(int iArgC, const char **ppcArgV) {
  int iSPS2Device;                 // Handle to the SPS2 device
  char *pcMemory;                  // DMA memory for transfer
  sps2Memory_t *pSPS2Memory;       // DMA memory descriptor
  Dn_CHCR_t chcrValue;             // Value sent to DMA controller to
                                   // initiate transfer
  Dn_SADR_t sadrValue;             // Destination scratch pad address
  Dn_MADR_t madrValue;             // Source memory address

  iSPS2Device=sps2Init();          // Initialize the SPS2 device

  if (iSPS2Device<0) {
    fprintf(stderr,"Error initializing SPS2 library\n");
    exit(-1);
  }

  // Allocate 4K of memory.  We're allocating in 4K chunks, we want this
  // memory to be cached this could improve performance if we did a lot
  // of work on this memory before transferring.  We have to remember to
  // flush the cache before the transfer
  pSPS2Memory=sps2Allocate(4096, SPS2_MAP_BLOCK_4K | SPS2_MAP_CACHED,
			    iSPS2Device);

  if (!pSPS2Memory) {
    fprintf(stderr, "Error allocating memory\n");
    exit(-2);
  }

  // Get the actual pointer to the memory
  pcMemory=(char *) pSPS2Memory->pvStart;

  // Copy our string to the memory
  strcpy(pcMemory, OUTPUT_STRING);

  // Flush the cache
  sps2FlushCache(iSPS2Device);

  // Set the memory address of the DMA transfer.  We're using channel
  // 9, which is a transfer to the scratch pad memory

  madrValue.i32=0;                  // Make sure all bits are zero
  // We're not copying from the scratch pad
  madrValue.s.SPR=0;
  // Get the physical address for the memory we allocated
  madrValue.s.ADDR=sps2GetPhysicalAddress(pcMemory, pSPS2Memory);
  // Set the memory address register in the DMA controller
  *EE_D9_MADR=madrValue.i32;

  sadrValue.i32=0;                  // Make sure all bits are zero
  // We're copying to the beginning of the scratch pad
  sadrValue.s.ADDR=0;
  // Set the scratch pad memory address in the DMA controller
  *EE_D9_SADR=sadrValue.i32;

  // Set the number of q-words to transfer.
  *EE_D9_QWC=OUTPUT_STRING_QWC;

  chcrValue.i32=0;                  // Set all CHCR bits to zero
  chcrValue.s.MOD=CHCR_MOD_NORMAL;  // Normal DMA transfer
  chcrValue.s.STR=1;                // Start DMA transfer

  // Set the CHCR register.  This starts the DMA transfer since we set STR=1
  *EE_D9_CHCR=chcrValue.i32;

  sps2WaitForDMA(9, iSPS2Device);  // Wait for DMA transfer to finish

  // Display the string now stored in the scratch pad
  printf((char *) SCRATCH_PAD);

  DPUT_GS_BGCOLOR(0xff0000);      // BG color is BBGGRR, set it to all blue

  sps2Release(iSPS2Device);       // Close the SPS2 library

  return 0;
}
